/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.azure;

import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.jclouds.azure.storage.AzureStorageServices;
import org.jclouds.blobstore.BlobStoreContext;
import org.jclouds.blobstore.BlobStoreContextFactory;
import org.jclouds.concurrent.MoreExecutors;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;

import com.google.inject.Module;

public class Azure extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(Azure.class);
    
    public Azure() { }

    @Override
    public String getCloudName() {
        return "Azure";
    }

    public BlobStoreContext getAzureContext() {
        logger.debug("enter - getAzureContext()");
        try {
            ProviderContext ctx = getContext();
            String account = new String(ctx.getAccessPublic());
            String key = new String(ctx.getAccessPrivate());
            
            ArrayList<Module> modules = new ArrayList<Module>();
            
            modules.add(new Log4JLoggingModule());
            modules.add(new ExecutorServiceModule(MoreExecutors.sameThreadExecutor(), MoreExecutors.sameThreadExecutor()));

            return (new BlobStoreContextFactory()).createContext("azureblob", account, key, modules);
        }
        finally {
            logger.debug("exit - getAzureContext()");
        }
    }
    
    @Override
    public DataCenterServices getDataCenterServices() {
        CloudProvider compute = getComputeCloud();
        
        return (compute == null ? new DataCenters(this) : compute.getDataCenterServices());
    }

    @Override
    public String getProviderName() {
        return "Microsoft";
    }

    @Override
    public AzureStorageServices getStorageServices() {
        return new AzureStorageServices(this);
    }
    
    @Override
    public String testContext() {
        try {
            getStorageServices().getBlobStoreSupport().listFiles(null).iterator().hasNext();
            return getContext().getAccountNumber();
        }
        catch( Throwable t ) {
            logger.warn("Failed to test Azure connection context: " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
